@@ -13,7 +13,7 @@ class action_plugin_fckg_edit extends DokuWiki_Action_Plugin {
     //store the namespaces for sorting
     var $fck_location = "fckeditor";
     var $helper       = false;
-
+    var $fckg_bak_file = "";
     /**
      * Constructor
      */
@@ -30,12 +30,38 @@ class action_plugin_fckg_edit extends DokuWiki_Action_Plugin {
 
     function register(&$controller)
     {
+     global $FCKG_show_preview;      
+     global  $Using_FCKG_EDITOR;  // TRUE if this an FCKG file or we are in the editor
+
+     $FCKG_show_preview=false;
+     list($ns, $b_no) = explode('.',  $_REQUEST['id']);    
+
+    $Using_FCKG_EDITOR = $this->isFCKG();  // if false, it is set to true in fckg editor
+
+     $b_no = isset($b_no) ? ++$b_no: 1;
+     $this->fckg_bak_file = $ns . ".$b_no";   // used in the edit page for check box title popup
   
+        if(isset($_REQUEST['fckg_bak']) && $_REQUEST['fckg_bak'] == 'fckg_bak') {
+          // create backups when switching from FCKG editor to Native DW editor
+          // backups are numbered incrementally
+     
+              $file = str_replace(':', '/', $_REQUEST['id']);
+              $orig = DOKU_INC . "data/pages/$file";              
+              $bak = str_replace(':', '/', $this->fckg_bak_file);  
+              $bak = DOKU_INC . "data/pages/$bak";    
+                    
+              $handle = $handle = fopen("$bak.txt", "w");
+              fwrite($handle,file_get_contents("$orig.txt"));
+              fclose($handle); 
+        }
+
         if(isset($_REQUEST['mode']) && $_REQUEST['mode'] == 'dwiki') {
            if(isset($_REQUEST['fck_convert'])) {
               $controller->register_hook('IO_WIKIPAGE_READ', 'BEFORE', $this, 'fck_convert_text');  
            }
-           return;
+
+          $FCKG_show_preview=true;
+          return;
         }
         elseif(isset($_COOKIE['FCKW_USE'])) {
              preg_match('/_\w+_/',  $_COOKIE['FCKW_USE'], $matches);
@@ -43,15 +69,40 @@ class action_plugin_fckg_edit extends DokuWiki_Action_Plugin {
                if(isset($_REQUEST['fck_convert'])) {
                   $controller->register_hook('IO_WIKIPAGE_READ', 'BEFORE', $this, 'fck_convert_text');  
                }
-
+                   $FCKG_show_preview=true;
                    return;
              }
         }
 
+        $Using_FCKG_EDITOR = true;
         $controller->register_hook('TPL_ACT_RENDER', 'BEFORE', $this, 'fckg_edit');
         $controller->register_hook('TPL_METAHEADER_OUTPUT', 'BEFORE', $this, 'fckg_edit_meta');
     }
 
+    function isFCKG() {
+      $file = str_replace(':', '/', $_REQUEST['id']);
+      $file = DOKU_INC . "data/pages/$file.txt";     
+      if(!file_exists($file)) {          
+           return true;
+      }
+      $text = file_get_contents($file);
+
+      $pos = strpos($text, '<align');    
+      if($pos !== false) {                    
+           return true;
+      }
+
+      $pos = strpos($text, '<fckg>');  
+      if ($pos === false) {
+           $pos = strpos($text, 'fckgCodeBLOCK'); 
+           if ($pos === false) {  
+               return false;
+           }
+      }
+      
+      return true;
+    }
+
     /**
      * fckg_edit_meta 
      *
@@ -72,6 +123,9 @@ class action_plugin_fckg_edit extends DokuWiki_Action_Plugin {
         global $ID;
         global $REV;
         global $INFO;
+        if(!isset($INFO['userinfo'])) {
+           return;
+        }
         global $WASSHOW;
         // load the text and check if it is a fck
         if($INFO['exists']){
@@ -213,7 +267,7 @@ class action_plugin_fckg_edit extends DokuWiki_Action_Plugin {
 // $temp=array();
 // trigger_event('HTML_EDITFORM_INJECTION', $temp);
 // print_r($_REQUEST);
-  
+$Using_FCKG_EDITOR = true;  
 ?>
 
    <form id="dw__editform" method="post" action="<?php echo script()?>" accept-charset="<?php echo $lang['encoding']?>"><div class="no">
@@ -287,13 +341,22 @@ $BIG_FILE_disabled="";
 if($this->big_file()) {
   $BIG_FILE_disabled='disabled';
 }
+
+$FCK_PREVIEW_DISPLAY = "";
+
+$fck_preview = $this->getConf('fck_preview');
+if(in_array($fck_preview, $INFO['userinfo']['grps'])) {  
+   $FCK_PREVIEW_DISPLAY = ' style="display:inline"; ';
+}
+
 ?>
 <input class="button" type="submit" name="do[draftdel]" value="<?php echo $lang['btn_cancel']?>" tabindex="8" />
-<!-- table width="100%" border=0><tr><td  -->
+
             <input class="button" id="edbtn__preview" type="submit" name="do[preview]"  
                      <?php echo $DW_EDIT_disabled; ?>
                      onmousedown="saveFCKEditorNode(this);return true;"  
                      <?php echo $BIG_FILE_disabled; ?>
+                     <?php echo $FCK_PREVIEW_DISPLAY; ?>
                      value="FCK Preview" accesskey="p" title="<?php echo $lang['btn_preview']?> 
                      [ALT+P]" tabindex="7" />
 
@@ -325,8 +388,8 @@ if($this->big_file()) {
                               <?php echo $DW_EDIT_disabled; ?>
                               onchange="document.getElementById('fck_convert').checked=false;"
                               onmouseup="document.getElementById('fck_convert').checked=false;"
-                              title="Your work will be lost"
-                             /> Discard  FCK and open blank editor&nbsp;&nbsp;&nbsp;&nbsp;
+                              title="Your work will be saved as <?php echo $this->fckg_bak_file ?>"
+                             /> Backup FCK and open blank editor&nbsp;&nbsp;&nbsp;&nbsp;
 
  <br/ ><!--  td-->
 <input type="checkbox" name="fckg_timer" value="fckg_timer"  id = "fckg_timer"
@@ -355,6 +418,39 @@ if($this->big_file()) {
         
            return true;
         }
+
+  var dwform = $('dw__editform');
+  var text = dwform.elements.wikitext.value;
+  var pre = text.indexOf('<pre');  
+  var table = text.indexOf('<table');  
+  if(pre != -1 || table != -1) {
+      text = text.replace(/\n/g, "__NL___");  // string replace balks at newlines
+      if(pre != -1) {
+            text = text.replace(/(<pre\s+class=\"code\">)(.*?)(<\/pre>)/g, repl_cb);
+      }
+      if(table != -1) {
+        text = text.replace(/(<td.*?>)(.*?)(<\/td>)/g, repl_cb);
+      }
+      text = text.replace(/__NL___/g,"\n");   // replace the newlines
+      dwform.elements.wikitext.value = text;
+  }
+
+function repl_cb(one,two,three,four) {
+// one is the entire match
+   var code = one.match(/<pre/) ? true : false;
+   three.replace(/_<(.*?)_>/g,"<$1>")
+   three = three.replace(/</g,'&lt;')
+   three = three.replace(/>/g,'&gt;')
+   three = three.replace(/&lt;([biu]{1,1})&gt;/g,'<$1>') 
+   three = three.replace(/&lt;\/([biu]{1,1})&gt;/g,'</$1>') 
+   if(!code) {
+       three = three.replace(/&lt;br \/&gt;/g,'<br />'); 
+       three = three.replace(/&lt;(strong|em)&gt;/g,'<$1>'); 
+       three = three.replace(/&lt;\/(strong|em)&gt;/g,'<\/$1>'); 
+   }
+   var str =  two + three + four;
+   return  str;
+}
   </script>
 
 
@@ -454,10 +550,10 @@ if($this->big_file()) {
      $text = preg_replace('/<font _dummy_\/.*?courier New>(.*?)<\/font>/ms', "''$1''", $text);  
      $text = preg_replace('/<align\s+left>(?!\\{2})\s*/ms', "\\\\\\\\  ", $text);  
      $text = preg_replace('/<align\s+0px>\s*/ms', "\\\\\\\\ ", $text);  
-     $text = preg_replace('/&nbsp;/ms', " ", $text);  //  remove &nbsp;
+     $text = preg_replace('/&nbsp;/ms', " ", $text);  //  remove &nbsp;     
      $text =  strip_tags($text, '<code>,<del>');
-
-    //     echo "<pre> text:" . htmlentities($text) . "</pre>";
+    
+   
      $text = preg_replace_callback(
           '/^\s+(.*?)$/ms',
            "trim_fck_convert_cb",
@@ -467,9 +563,21 @@ if($this->big_file()) {
 
     $line_break = '\\\\\\\\';
     $text = preg_replace ('/(' . $line_break . '[\s\n]+)+/', "$line_break\n", $text);
-  //  $text .= "REQUEST: " . $_REQUEST['fck_convert'];
-   //   echo "<pre> text:" . htmlentities($text) . "</pre>";
-      io_SaveFile($event->data[0][0], $text);  
+
+     $text = preg_replace_callback(
+          '/<code>(.*?)<\/code>/ms',
+           "trim_code_escapes",
+          $text
+      );  
+
+     // process td's
+     $text = preg_replace_callback(
+          '/\|(.*?)\|/ms',
+           "trim_code_escapes",
+          $text
+      );  
+    
+    io_SaveFile($event->data[0][0], $text);  
 
   }
 
@@ -530,4 +638,15 @@ function trim_fck_align_cell($align, $str) {
         return $text . "\n";
   }
 
+  function trim_code_escapes($matches) {
+     $text = $matches[0];  
+     $text = str_replace('fckgOpen_C_COMMENT', '/*', $text);
+     $text = str_replace('fckgClosed_C_COMMENT', '*/', $text);
+     $text = str_replace('fckgCodeBLOCK', "", $text);
+     $text = str_replace('fckgOpenPAREN', '<', $text);    
+     $text = str_replace("%%", "", $text);
+     $text = str_replace('fckgTableOpenBRACKET', "<", $text);     
+
+     return $text;
+  }
 ?>
